#include <stdlib.h>
#include <string.h>
#include <math.h>
#ifdef _MSC_VER
#include "SDL.h"
#else
#include "SDL/SDL.h"
#endif
#include "gp.h"

void collectiblecollision()
{
  int i;
  for (i = 0; i < gCollectibleCount; i++)
  {
    if (gCollectible[i].mTaken == 0)
    {
      if (sqrt((gCollectible[i].mX - gXPos) * 
               (gCollectible[i].mX - gXPos) +
               (gCollectible[i].mY - gYPos) * 
               (gCollectible[i].mY - gYPos)) <
          RADIUS + gCollectible[i].mRadius)
      {
        gCollectiblesTaken++;
        gCollectible[i].mTaken = 1;
      }
    }
  }
}

void reset()
{
  gXMov = 0;
  gYMov = 0;
  gRoll = 0;

  gKeyLeft = 0;
  gKeyRight = 0;
  gKeyUp = 0;
  gKeyDown = 0;

  char name[80];
  FILE * f;
  do
  {
    sprintf(name, "level%d.txt", gCurrentLevel);
    f = fopen(name, "rb");
    if (f == NULL)
    {
      if (gCurrentLevel == 0)
        exit(0);
      gCurrentLevel = 0;
    }
  }
  while (f == NULL);

  gLevelWidth = -1;
  int i = 0;
  while (i != '\n' && i != '\r')
  {
    i = fgetc(f);
    gLevelWidth++;
  }
  gLevelHeight = 1;
  while (!feof(f))
  {
    if (fgetc(f) == i)
      gLevelHeight++;
  }
  fseek(f, 0, SEEK_SET);

  delete[] gLevel;
  gLevel = new unsigned char[gLevelWidth * gLevelHeight];

  int p = 0;
  while (p < gLevelWidth * gLevelHeight && !feof(f))
  {
    int v = fgetc(f);
    if (v > 32)
    {
      switch (v)
      {
      case '.':
        gLevel[p] = LEVEL_DROP;
        break;
      case '_':
        gLevel[p] = LEVEL_GROUND;
        break;
      case 'S':
        gLevel[p] = LEVEL_START;
        break;
      case 'E':
        gLevel[p] = LEVEL_END;
        break;
      case 'o':
        gLevel[p] = LEVEL_COLLECTIBLE;
        break;
      case '>':
        gLevel[p] = LEVEL_RIGHT;
        break;
      case '<':
        gLevel[p] = LEVEL_LEFT;
        break;
      case 'v':
        gLevel[p] = LEVEL_DOWN;
        break;
      case '^':
        gLevel[p] = LEVEL_UP;
        break;
      }
      p++;
    }
  }
  fclose(f);

  gCollectibleCount = 0;
  for (i = 0; i < gLevelWidth * gLevelHeight; i++)
  {
    if (gLevel[i] == LEVEL_START)
    {
      gStartX = (float)((i % gLevelWidth) * TILESIZE + TILESIZE / 2);
      gStartY = (float)((i / gLevelWidth) * TILESIZE + TILESIZE / 2);
    }
    if (gLevel[i] == LEVEL_COLLECTIBLE)
      gCollectibleCount++;
  }
  
  delete[] gCollectible;
  gCollectible = new collectible[gCollectibleCount];
  gCollectibleCount = 0;
  for (i = 0; i < gLevelWidth * gLevelHeight; i++)
  {
    if (gLevel[i] == LEVEL_COLLECTIBLE)
    {
      gCollectible[gCollectibleCount].mX = 
        (float)((i % gLevelWidth) * TILESIZE + TILESIZE / 2);
      gCollectible[gCollectibleCount].mY = 
        (float)((i / gLevelWidth) * TILESIZE + TILESIZE / 2);
      gCollectible[gCollectibleCount].mColor = COLLECTIBLECOLOR;
      gCollectible[gCollectibleCount].mRadius = COLLECTIBLERADIUS;
      gCollectible[gCollectibleCount].mTaken = 0;
      gCollectibleCount++;
    }
  }

  memcpy(gRVtx, gVtx, sizeof(vertex) * BALLVTXCOUNT);

  gCollectiblesTaken = 0;

  gXPos = gStartX;
  gYPos = gStartY;

  gCameraX = (WIDTH / 2) - gXPos;
  gCameraY = (HEIGHT / 2) - gYPos;

  gLastTick = SDL_GetTicks(); 
  gLevelStartTick = SDL_GetTicks(); 
}

void rendergame()
{   
  // Ask SDL for the time in milliseconds
  int tick = SDL_GetTicks();

  if (tick <= gLastTick) 
  {
    SDL_Delay(1);
    return;
  }

  while (gLastTick < tick)
  {
    if (gKeyLeft) gXMov -= THRUST;
    if (gKeyRight) gXMov += THRUST;
    if (gKeyUp) gYMov -= THRUST;
    if (gKeyDown) gYMov += THRUST;

    if (gJoystick)
    {
      gXMov += (SDL_JoystickGetAxis(gJoystick, 0) / 32768.0f) * THRUST;
      gYMov += (SDL_JoystickGetAxis(gJoystick, 1) / 32768.0f) * THRUST;
    }

    switch (gLevel[(((int)gYPos) / TILESIZE) * gLevelWidth + ((int)gXPos) / TILESIZE])
    {
    case LEVEL_DROP:
      // player fell off - reset position
      reset();
      break;
    case LEVEL_END:
      gScore += 100;
      if (gCollectibleCount > 0)
      {
        int secondsleft = TIMELIMIT - (gLastTick - gLevelStartTick) / 1000;
        if (secondsleft < 0)
          secondsleft = 0;
        gScore += (secondsleft * 20) * gCollectiblesTaken / gCollectibleCount;
      }
      gCurrentLevel++;
      reset();
      break;
    case LEVEL_LEFT:
      gXMov -= SLIDEPOWER;
      break;
    case LEVEL_RIGHT:
      gXMov += SLIDEPOWER;
      break;
    case LEVEL_UP:
      gYMov -= SLIDEPOWER;
      break;
    case LEVEL_DOWN:
      gYMov += SLIDEPOWER;
      break;
    }

    gXMov *= SLOWDOWN;
    gYMov *= SLOWDOWN;

    gXPos += gXMov;
    gYPos += gYMov;

    float targetx = (WIDTH / 2) - (gXPos + gXMov * 25);
    float targety = (HEIGHT / 2) - (gYPos + gYMov * 25);
    gCameraX = (targetx + gCameraX * 19) / 20;
    gCameraY = (targety + gCameraY * 19) / 20;

    gRoll += (float)sqrt(gXMov * gXMov + gYMov * gYMov);

    // Collision with the level borders
    if (gXPos > gLevelWidth * TILESIZE || 
        gXPos < 0 || 
        gYPos > gLevelHeight * TILESIZE || 
        gYPos < 0)
    {
      reset();
    }

    collectiblecollision();

    gLastTick += 1000 / PHYSICSFPS;
  }

  // Lock surface if needed
  if (SDL_MUSTLOCK(gScreen))
    if (SDL_LockSurface(gScreen) < 0) 
      return;

  // fill background
  drawbackground(tick, 
                 (int)(WIDTH * ((WIDTH / 2) - gCameraX) / (gLevelWidth * TILESIZE)), 
                 (int)(HEIGHT * ((HEIGHT / 2) - gCameraY) / (gLevelHeight * TILESIZE)));
  int i, j;
  for (i = 0; i < gLevelHeight; i++)
  {
    for (j = 0; j < gLevelWidth; j++)
    {
            if (gLevel[i * gLevelWidth + j] != 0)
      {
        int tile = 0;

        switch (gLevel[i * gLevelWidth + j])
        {
        case LEVEL_START:
          tile = 2;
          break;
        case LEVEL_END:
          tile = 1;
          break;
        case LEVEL_UP:
          tile = 4;
          break;
        case LEVEL_RIGHT:
          tile = 5;
          break;
        case LEVEL_DOWN:
          tile = 6;
          break;
        case LEVEL_LEFT:
          tile = 7;
          break;
        case LEVEL_GROUND:
        case LEVEL_COLLECTIBLE:
        default:
          // tile = 0;
          break;
        }
        drawtile((int)(j * TILESIZE + gCameraX), 
                 (int)(i * TILESIZE + gCameraY), tile);
      }
    }
  }

  // draw the collectibles
  for (i = 0; i < gCollectibleCount; i++)
  {
    if (gCollectible[i].mTaken == 0)
    {
      drawcircle((int)(gCollectible[i].mX + 2 + gCameraX),
             (int)(gCollectible[i].mY + 2 + gCameraY),
             gCollectible[i].mRadius,
             0);
      drawcircle((int)(gCollectible[i].mX + gCameraX),
             (int)(gCollectible[i].mY + gCameraY),
             gCollectible[i].mRadius,
             gCollectible[i].mColor);
    }
  }

  // draw the player object
  drawball((int)(gXPos + gCameraX), (int)(gYPos + gCameraY), 
           RADIUS, BALLCOLOR, BALLLOCOLOR, BALLHICOLOR, 
           gRoll/10.0f, (float)atan2(gYMov, gXMov));

  gRoll = 0;

  char scorestring[80];
  sprintf(scorestring, "Score:%d", gScore);
  drawstring(5, 5, scorestring);

  int secondsleft = TIMELIMIT - (gLastTick - gLevelStartTick) / 1000;
  if (secondsleft < 0)
    secondsleft = 0;
  sprintf(scorestring, "Time:%d", secondsleft);
  drawstring(5, 22, scorestring);

  // Unlock if needed
    if (SDL_MUSTLOCK(gScreen)) 
        SDL_UnlockSurface(gScreen);

  // Tell SDL to update the whole gScreen
    SDL_UpdateRect(gScreen, 0, 0, WIDTH, HEIGHT);    
}
